/*****************************************************************************/
/*                      WiNRADiO G69DDC EXCELSIOR ULTRA                      */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2022                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_global.h"


enum
{
    ID_POWER=wxID_HIGHEST,
    ID_SPECTRUM,
    ID_SWEEPER,
    ID_IF_DOWN,
    ID_IF_UP,
    ID_SWP_SPECTRUM,
    ID_SWP_START_FREQ,
    ID_SWP_END_FREQ,
    ID_SWP_START,
    ID_SWP_STOP,
    ID_SWP_LOOP,
    ID_SWP_CLEAR,
    ID_ATTENUATOR,
    ID_PREAMP,
    ID_MW_FILTER,
    ID_IFRBW,
    ID_IF_SPECTRUM,
    ID_STATE_TIMER,
    ID_INIT_POWER_ON_TIMER,
    ID_DDC1_SPECTRUM,
    ID_DDC1BW,
    ID_DDC1RBW,
    ID_RX_BASE=wxID_HIGHEST+100
};

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_STOP_SWEEPER,5004)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_COMMAND_STOP_SWEEPER)

#define EVT_STOP_SWEEPER(fn)                                        \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_STOP_SWEEPER, -1, -1,   \
	(wxObjectEventFunction)(wxEventFunction)                        \
	(wxCommandEventFunction) & fn, (wxObject*) NULL ),

BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_IF_DOWN,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_IF_UP,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_PREAMP,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_MW_FILTER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWEEPER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SPECTRUM,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWP_START,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWP_LOOP,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_SWP_STOP,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_SWP_CLEAR,TMainWindow::OnButtonClick)
    EVT_COMBOBOX(ID_IFRBW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ATTENUATOR,TMainWindow::OnComboBoxChanged)
    EVT_SPECTRUM_TUNE(ID_IF_SPECTRUM,TMainWindow::OnSpectrumTune)
    EVT_SPECTRUM_TUNE(ID_SWP_SPECTRUM,TMainWindow::OnSpectrumTune)
    EVT_TIMER(ID_STATE_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_INIT_POWER_ON_TIMER,TMainWindow::OnTimer)
    EVT_COMBOBOX(ID_DDC1BW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DDC1RBW,TMainWindow::OnComboBoxChanged)
    EVT_SPECTRUM_TUNE(ID_DDC1_SPECTRUM,TMainWindow::OnSpectrumTune)
    EVT_STOP_SWEEPER(TMainWindow::OnStopSweeper)
    EVT_NUMERICEDIT(ID_SWP_START_FREQ,TMainWindow::OnNumericEditChanged)
    EVT_NUMERICEDIT(ID_SWP_END_FREQ,TMainWindow::OnNumericEditChanged)
END_EVENT_TABLE()


TMainWindow::TMainWindow(TReceiver *Receiver):wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,800)),
m_Receiver(Receiver),m_StateTimer(this,ID_STATE_TIMER),m_InitPowerOnTimer(this,ID_INIT_POWER_ON_TIMER)
{
 uint32_t i;
 wxString Title,s;
 G69DDC_DEVICE_STATE DeviceState;
 wxString Intf;

    m_ActiveRXPanel=NULL;
    m_SwpLoop=true;

    m_DeviceInfo=Receiver->GetDeviceInfo();

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    Title.sprintf(APP_TITLE wxT(" - %s"),wxString(m_DeviceInfo.SerialNumber,wxConvLocal).c_str());

    switch(m_DeviceInfo.InterfaceType)
    {
        case G69DDC_INTERFACE_TYPE_USB2:
            Title+=wxT(" / USB2");
            break;

        case G69DDC_INTERFACE_TYPE_USB3:
            Title+=wxT(" / USB3");
            break;

        case G69DDC_INTERFACE_TYPE_PCIE:
            Title+=wxT(" / PCIe");
            break;

        case G69DDC_INTERFACE_TYPE_LAN:
            Title+=wxT(" / LAN");
            break;

        default:
            break;
    }

    SetTitle(Title);

    m_MainSizer=new wxBoxSizer(wxVERTICAL);

    m_IFSizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_IFSizerBottom=new wxBoxSizer(wxHORIZONTAL);

    m_DDC1Sizer=new wxBoxSizer(wxVERTICAL);
    m_DDC1SizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_DDC1Sizer->Add(m_DDC1SizerTop,0,wxEXPAND|wxALIGN_TOP|wxBOTTOM,4);

    m_RXSizer=new wxBoxSizer(wxHORIZONTAL);
    m_MainSizer->Add(m_RXSizer,0,wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP|wxALL,4);

    m_ChannelCount=m_Receiver->GetChannelCount();

    m_BRX=new wxToggleButton*[m_ChannelCount];

    m_RXPanels=new TRXPanel*[m_ChannelCount];

    for(i=0;i<m_ChannelCount;i++)
    {
        s.sprintf(wxT("RX %u"),i+1);
        m_BRX[i]=new wxToggleButton(this,ID_RX_BASE+i,s,wxDefaultPosition,wxSize(47,23));
        m_RXSizer->Add(m_BRX[i],0,wxRIGHT,4);

        m_RXPanels[i]=new TRXPanel(this,m_Receiver,i);
        m_MainSizer->Add(m_RXPanels[i],1,wxEXPAND|wxLEFT|wxRIGHT,4);

        if(i!=0)
        {
            m_RXPanels[i]->Hide();
        }
    }

    Connect(ID_RX_BASE,ID_RX_BASE+m_ChannelCount-1,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,wxCommandEventHandler(TMainWindow::OnRXButtonClick));

    m_MainSizer->Add(m_DDC1Sizer,0,wxALIGN_TOP|wxEXPAND|wxRIGHT|wxLEFT|wxTOP,4);

    m_LDDC1BW=new wxStaticText(this,wxID_ANY,wxT("DDC1 BW"));
    m_DDC1SizerTop->Add(m_LDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1BW=new wxComboBox(this,ID_DDC1BW,wxEmptyString,wxDefaultPosition,wxSize(100,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_DDC1SizerTop->Add(m_CBDDC1BW,0,wxALIGN_LEFT);

    m_DDC1SizerTop->AddSpacer(8);

    m_LDDC1RBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_DDC1SizerTop->Add(m_LDDC1RBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1RBW=new wxComboBox(this,ID_DDC1RBW,wxEmptyString,wxDefaultPosition,wxSize(100,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_DDC1SizerTop->Add(m_CBDDC1RBW,0,wxALIGN_LEFT);

    m_DDC1Spectrum=new TSpectrum(this,ID_DDC1_SPECTRUM,wxDefaultPosition,wxSize(100,160));
    m_DDC1Spectrum->SetMinLevel(-160);
    m_DDC1Spectrum->SetBand(SpectrumBand_BothSides,true);
    m_DDC1Spectrum->SetText(wxT("DDC1 spectrum"));
    m_DDC1Sizer->Add(m_DDC1Spectrum,1,wxEXPAND);

    m_MainSizer->Add(m_IFSizerTop,0,wxALIGN_TOP|wxALL|wxEXPAND,4);

    m_BSpectrum=new wxToggleButton(this,ID_SPECTRUM,wxT("Spectrum"),wxDefaultPosition,wxSize(65,23));
    m_IFSizerTop->Add(m_BSpectrum,0,wxALIGN_LEFT);

    m_IFSizerTop->AddSpacer(4);

    m_BSweeper=new wxToggleButton(this,ID_SWEEPER,wxT("Sweeper"),wxDefaultPosition,wxSize(65,23));
    m_IFSizerTop->Add(m_BSweeper,0,wxALIGN_LEFT);

    m_IFSizerTop->AddSpacer(8);

    m_LRBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_IFSizerTop->Add(m_LRBW,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    m_IFSizerTop->AddSpacer(4);

    m_CBIFRBW=new wxComboBox(this,ID_IFRBW,wxEmptyString,wxDefaultPosition,wxSize(100,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_IFSizerTop->Add(m_CBIFRBW,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);


    m_CBSwpRBW=new wxComboBox(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(100,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_IFSizerTop->Add(m_CBSwpRBW,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_LSwpStart=new wxStaticText(this,wxID_ANY,wxT("Start frequency"));
    m_IFSizerTop->Add(m_LSwpStart,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_ESwpStart=new TFrequencyEdit(this,ID_SWP_START_FREQ,10000000,0,m_DeviceInfo.Range2.MaxFrequency+m_DeviceInfo.Range2.Bandwidth/2,wxDefaultPosition,wxSize(105,23));
    m_IFSizerTop->Add(m_ESwpStart,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_LSwpEnd=new wxStaticText(this,wxID_ANY,wxT("End frequency"));
    m_IFSizerTop->Add(m_LSwpEnd,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_ESwpEnd=new TFrequencyEdit(this,ID_SWP_END_FREQ,100000000,1000000,m_DeviceInfo.Range2.MaxFrequency+m_DeviceInfo.Range2.Bandwidth/2,wxDefaultPosition,wxSize(105,23));
    m_IFSizerTop->Add(m_ESwpEnd,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_BSwpStart=new wxToggleButton(this,ID_SWP_START,wxT("Start"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpStart,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_BSwpStop=new wxButton(this,ID_SWP_STOP,wxT("Stop"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpStop,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_BSwpLoop=new wxToggleButton(this,ID_SWP_LOOP,wxT("Loop"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpLoop,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_BSwpClear=new wxButton(this,ID_SWP_CLEAR,wxT("Clear"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpClear,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    m_LDataRate=new wxStaticText(this,wxID_ANY,wxT("Data rate"));
    m_IFSizerTop->Add(m_LDataRate,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    m_LDataRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(60,-1),wxALIGN_RIGHT);
    m_IFSizerTop->Add(m_LDataRateValue,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT,4);

    m_LLostDataRate=new wxStaticText(this,wxID_ANY,wxT("Lost data rate"));
    m_IFSizerTop->Add(m_LLostDataRate,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT,8);

    m_LLostDataRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(60,-1),wxALIGN_RIGHT);
    m_IFSizerTop->Add(m_LLostDataRateValue,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxLEFT,4);


    m_IFSizerTop->AddStretchSpacer();

    m_LFrontEndFrequency=new wxStaticText(this,wxID_ANY,wxT("Front-end frequency"));
    m_IFSizerTop->Add(m_LFrontEndFrequency,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_LFrontEndFrequencyValue=new wxStaticText(this,wxID_ANY,wxT("0 MHz"),wxDefaultPosition,wxSize(55,-1),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    m_IFSizerTop->Add(m_LFrontEndFrequencyValue,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);


    m_LRange=new wxStaticText(this,wxID_ANY,wxT("Range"));
    m_IFSizerTop->Add(m_LRange,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,4);

    m_LRangeValue=new wxStaticText(this,wxID_ANY,wxT("1"),wxDefaultPosition,wxSize(12,-1),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    m_IFSizerTop->Add(m_LRangeValue,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_BIFUp=new wxButton(this,ID_IF_UP,wxT("+"),wxDefaultPosition,wxSize(40,23));
    m_IFSizerTop->Add(m_BIFUp,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,2);

    m_BIFDown=new wxButton(this,ID_IF_DOWN,wxT("-"),wxDefaultPosition,wxSize(40,23));
    m_IFSizerTop->Add(m_BIFDown,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,8);

    m_IFSpectrum=new TSpectrum(this,ID_IF_SPECTRUM,wxDefaultPosition,wxSize(100,160));
    m_IFSpectrum->SetText(wxT("IF spectrum"));
    m_IFSpectrum->SetMinLevel(-140);
    m_MainSizer->Add(m_IFSpectrum,0,wxEXPAND|wxLEFT|wxRIGHT,4);

    m_SwpSpectrum=new TSwpSpectrum(this,ID_SWP_SPECTRUM,wxDefaultPosition,wxSize(100,160));
    m_MainSizer->Add(m_SwpSpectrum,0,wxEXPAND|wxLEFT|wxRIGHT,4);

    m_MainSizer->Add(m_IFSizerBottom,0,wxALIGN_BOTTOM|wxEXPAND|wxALL,4);

    m_IFSizerBottom->AddSpacer(4);

    m_LAttenuator=new wxStaticText(this,wxID_ANY,wxT("Attenuator"));
    m_IFSizerBottom->Add(m_LAttenuator,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_IFSizerBottom->AddSpacer(4);

    m_CBAttenuator=new wxComboBox(this,ID_ATTENUATOR,wxEmptyString,wxDefaultPosition,wxSize(90,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_IFSizerBottom->Add(m_CBAttenuator,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddSpacer(4);

    m_BPreamp=new wxToggleButton(this,ID_PREAMP,wxT("Preamplifier"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPreamp,0,wxALIGN_LEFT);

    m_BMWFilter=new wxToggleButton(this,ID_MW_FILTER,wxT("MW Filter"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BMWFilter,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddStretchSpacer();

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPower,0,wxRIGHT);


    SetSizer(m_MainSizer);


    Initialize();

    SetClientSize(878,750);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    UpdateBottomPanel();

    m_BSwpLoop->SetValue(m_SwpLoop);

    //set receiver callbacks to this object
    //it is done after Initialize() to prevent from many calls of TReceiver methods
    //during initialization of controls which could invoke many receiver/channel events
    m_Receiver->SetCallback(this);

    m_StateTimer.Start(500);

    //power on with delay
    m_InitPowerOnTimer.Start(200,true);

    Layout();
    Centre();

    m_Receiver->GetDeviceState(&DeviceState);

    m_StopWatch.Start();
    m_Time=m_StopWatch.TimeInMicro();
    m_BytesTransferred=DeviceState.DataTransferred;
    m_BytesLost=DeviceState.DataLost;
}

TMainWindow::~TMainWindow(void)
{
    delete m_Receiver;

    delete[] m_RXPanels;
    delete[] m_BRX;
}

//ReceiverCallback_Event is called when a receiver event occurs (e.g. power state changed, DDC1 frequency changed,...)
void TMainWindow::ReceiverCallback_Event(TReceiverEvent Event)
{
 int64_t Start,End;
 double RBW;

    switch(Event)
    {
        case ReceiverEvent_Power:
            UpdateState();
            break;

        case ReceiverEvent_Attenuator:
            UpdateAttenuatorState();
            break;

        case ReceiverEvent_Preamplifier:
            UpdatePreamplifier();
            break;

        case ReceiverEvent_Range:
            UpdateRange();
            break;

        case ReceiverEvent_FrontEndFrequency:
            UpdateFrontEndFrequency();
            break;

        case ReceiverEvent_IFRBWIndex:
            UpdateIFRBWIndex();
            break;

        case ReceiverEvent_ActiveChannel:
            UpdateActiveChannel();
            break;

        case ReceiverEvent_DDC1:
            UpdateDDC1();
            break;

        case ReceiverEvent_DDC1_AbsFrequency:
            UpdateDDC1AbsFrequency();
            break;

        case ReceiverEvent_DDC1_RBWList:
            UpdateDDC1RBWList();
            break;

        case ReceiverEvent_DDC1_RBWIndex:
            UpdateDDC1RBW();
            break;

        case ReceiverEvent_SweeperStarted:
            m_Receiver->GetSweeperParams(&Start,&End,&RBW);
            m_SwpSpectrum->Start(Start,End,RBW);
            UpdateState();
            break;

        case ReceiverEvent_SweeperStopped:
            m_SwpSpectrum->Stop();
            UpdateState();
            break;

        default:
            break;
    }
}

void TMainWindow::ReceiverCallback_ChannelEvent(uint32_t Channel,TChannelEvent Event)
{
    switch(Event)
    {
        case ChannelEvent_DDC2_Frequency:
            UpdateDDC2Frequency(Channel);
            break;

        case ChannelEvent_DDC2:
            UpdateDDC2(Channel);
            break;

        default:
            break;
    }

    //forward event to RXPanel
    m_RXPanels[Channel]->ReceiverCallback_ChannelEvent(Event);
}


void TMainWindow::Initialize(void)
{
 uint32_t i,Count;
 G69DDC_DDC_INFO DDCInfo;
 int RBW;
 wxString s;
 double RBWs[RBW_COUNT];


    m_Receiver->GetIFRBWs(RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=(int)(RBWs[i]+0.5);

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        s=wxT(" ")+FormatFrequency(RBW);

        m_CBIFRBW->Append(s);
        m_CBSwpRBW->Append(s);
    }

    m_CBSwpRBW->SetSelection(1);

    m_CBAttenuator->Append(wxT(" Off"));
    m_CBAttenuator->Append(wxT(" 3 dB"));
    m_CBAttenuator->Append(wxT(" 6 dB"));
    m_CBAttenuator->Append(wxT(" 9 dB"));
    m_CBAttenuator->Append(wxT(" 12 dB"));
    m_CBAttenuator->Append(wxT(" 15 dB"));
    m_CBAttenuator->Append(wxT(" 18 dB"));
    m_CBAttenuator->Append(wxT(" 21 dB"));


    m_IFSpectrum->SetBand(SpectrumBand_BothSides,false);

    UpdateRange();
    UpdateFrontEndFrequency();
    UpdateAttenuatorState();
    UpdatePreamplifier();
    UpdateMWFilter();
    UpdateIFRBWIndex();


    Count=m_Receiver->GetDDC1Count();

    for(i=0;i<Count;i++)
    {
        m_Receiver->GetDDCInfo(i,&DDCInfo);

        m_CBDDC1BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));
    }

    for(i=0;i<m_ChannelCount;i++)
    {
        m_RXPanels[i]->Initialize();
    }

    UpdateDDC1();
    UpdateDDC1AbsFrequency();
    UpdateDDC1RBWList();

    UpdateState();

    UpdateActiveChannel();

    UpdateDDC2(m_Receiver->GetActiveChannel());

    m_BSpectrum->SetValue(true);
}

void TMainWindow::OnRXButtonClick(wxCommandEvent &Event)
{
 uint32_t Channel;

    Channel=Event.GetId()-ID_RX_BASE;

    if(Channel==m_Receiver->GetActiveChannel())
    {
        m_BRX[Channel]->SetValue(true);
    }
    else
    {
        m_Receiver->SetActiveChannel(Channel);
    }
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER:
            BusyCursor=new wxBusyCursor;

            m_BPower->Update();

            m_Receiver->SetPower(m_BPower->GetValue());

            delete BusyCursor;
            break;

        case ID_PREAMP:
            m_Receiver->SetPreamplifier(m_BPreamp->GetValue());
            break;

        case ID_MW_FILTER:
            m_Receiver->SetMWFilter(m_BMWFilter->GetValue());
            break;

        case ID_SPECTRUM:
            if(!m_BSpectrum->GetValue())
            {
                m_BSpectrum->SetValue(true);
                return;
            }
            m_BSweeper->SetValue(false);
            UpdateBottomPanel();
            break;

        case ID_SWEEPER:
            if(!m_BSweeper->GetValue())
            {
                m_BSweeper->SetValue(true);
                return;
            }
            m_BSpectrum->SetValue(false);
            UpdateBottomPanel();
            break;

        case ID_IF_DOWN:
            m_Receiver->FrontEndFrequencyDown();
            break;

        case ID_IF_UP:
            m_Receiver->FrontEndFrequencyUp();
            break;

        case ID_SWP_START:
            if(m_BSwpStart->GetValue())
            {
                if(!m_Receiver->StartSweeper(m_ESwpStart->Get(),m_ESwpEnd->Get(),m_CBSwpRBW->GetSelection()))
                {
                    m_BSwpStart->SetValue(false);
                }
            }
            else
            {
                m_BSwpStart->SetValue(true);
            }
            break;

        case ID_SWP_STOP:
            m_Receiver->StopSweeper();
            break;

        case ID_SWP_CLEAR:
            m_SwpSpectrum->Clear();
            break;

        case ID_SWP_LOOP:
            m_SwpLoop=m_BSwpLoop->GetValue();
            break;
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_DDC1BW:
            if(m_CBDDC1BW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1(m_CBDDC1BW->GetSelection());
            }
            break;

        case ID_DDC1RBW:
            if(m_CBDDC1RBW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1RBWIndex(m_CBDDC1RBW->GetSelection());
            }
            break;

        case ID_ATTENUATOR:
            m_Receiver->SetAttenuator(m_CBAttenuator->GetSelection());
            break;

        case ID_IFRBW:
            m_Receiver->SetIFRBWIndex(m_CBIFRBW->GetSelection());
            break;
    }
}

void TMainWindow::UpdateState(void)
{
 bool Power;
 bool Enabled;
 bool Connected;
 uint32_t i;

    Power=m_Receiver->GetPower();
    Connected=m_Receiver->IsConnected();

    Enabled=Power && Connected && !m_Receiver->GetSweeper();

    for(i=0;i<m_ChannelCount;i++)
    {
        m_BRX[i]->Enable(Enabled);
    }

    m_BPower->SetValue(Power);
    m_IFSpectrum->Enable(Enabled);
    m_IFSpectrum->Clear();

    m_LRBW->Enable(Enabled);
    m_CBIFRBW->Enable(Enabled);

    m_LDDC1BW->Enable(Enabled);
    m_CBDDC1BW->Enable(Enabled);
    m_LDDC1RBW->Enable(Enabled);
    m_CBDDC1RBW->Enable(Enabled);

    m_DDC1Spectrum->Enable(Enabled);

    UpdateAttenuatorState();

    m_BPreamp->Enable(Enabled);
    m_BMWFilter->Enable(Enabled);

    m_LDataRate->Enable(Enabled);
    m_LDataRateValue->Enable(Enabled);
    m_LLostDataRate->Enable(Enabled);
    m_LLostDataRateValue->Enable(Enabled);
    m_LFrontEndFrequency->Enable(Enabled);
    m_LFrontEndFrequencyValue->Enable(Enabled);
    m_LRange->Enable(Enabled);
    m_LRangeValue->Enable(Enabled);
    m_BIFUp->Enable(Enabled);
    m_BIFDown->Enable(Enabled);

    for(i=0;i<m_ChannelCount;i++)
    {
        m_RXPanels[i]->UpdateState();
    }

    if(!Enabled)
    {
        m_DDC1Spectrum->Clear();
        m_IFSpectrum->Clear();
    }

    UpdateSwpState();
}



void TMainWindow::UpdateAttenuatorState(void)
{
 bool Enabled;

    m_CBAttenuator->SetSelection(m_Receiver->GetAttenuator());

    Enabled=m_Receiver->GetPower();

    m_CBAttenuator->Enable(Enabled);
    m_LAttenuator->Enable(Enabled);
}

void TMainWindow::UpdatePreamplifier(void)
{
    m_BPreamp->SetValue(m_Receiver->GetPreamplifier());
}

void TMainWindow::UpdateMWFilter(void)
{
    m_BMWFilter->SetValue(m_Receiver->GetMWFilter());
}

void TMainWindow::ReceiverCallback_IFSpectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_IFSpectrum->SetData(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_DDC1Spectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_DDC1Spectrum->SetData(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_DDC2Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)
{
    m_RXPanels[Channel]->ReceiverCallback_DDC2Spectrum(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_AudioSpectrum(uint32_t Channel,float *Data,uint32_t Count,uint32_t Range)
{
    m_RXPanels[Channel]->ReceiverCallback_AudioSpectrum(Data,Count,Range);
}

void TMainWindow::UpdateIFRBWIndex(void)
{
    m_CBIFRBW->SetSelection(m_Receiver->GetIFRBWIndex());
}

void TMainWindow::ReceiverCallback_Audio(uint32_t Channel,const float *Samples,uint32_t Count)
{
    m_RXPanels[Channel]->ReceiverCallback_Audio(Samples,Count);
}

void TMainWindow::UpdateActiveChannel(void)
{
 uint32_t Channel,i;
 TRXPanel *Panel;

    Channel=m_Receiver->GetActiveChannel();

    if(m_ActiveRXPanel!=m_RXPanels[Channel])
    {
        for(i=0;i<m_ChannelCount;i++)
        {
            m_BRX[i]->SetValue(i==Channel);
        }

        Panel=m_ActiveRXPanel;

        m_ActiveRXPanel=m_RXPanels[Channel];
        m_ActiveRXPanel->Show(true);

        if(Panel)
        {
            Panel->Show(false);
        }

        UpdateDDC2Frequency(Channel);

        Layout();
    }
}

void TMainWindow::UpdateDDC1AbsFrequency(void)
{
    m_DDC1Spectrum->SetCenterFrequency(m_Receiver->GetDDC1AbsFrequency());
    m_IFSpectrum->SetBandFrequency(m_Receiver->GetDDC1AbsFrequency());
}

void TMainWindow::UpdateDDC1(void)
{
    m_IFSpectrum->SetBandWidth(m_Receiver->GetDDC1Info().Bandwidth);

    m_CBDDC1BW->SetSelection(m_Receiver->GetDDC1());

    m_DDC1Spectrum->SetRange(m_Receiver->GetDDC1Info().Bandwidth);
}

void TMainWindow::OnSpectrumTune(TSpectrumTuneEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_DDC1_SPECTRUM:
            m_Receiver->SetDDC2Frequency(m_Receiver->GetActiveChannel(),(int32_t)Event.GetFrequency());
            break;

        case ID_IF_SPECTRUM:
            m_Receiver->SetDemAbsFrequency(m_ActiveRXPanel->GetChannel(),Event.GetFrequency());
            break;

        case ID_SWP_SPECTRUM:
            m_Receiver->SetDemAbsFrequency(m_ActiveRXPanel->GetChannel(),Event.GetFrequency());
            break;
    }
}

wxString TMainWindow::DataRateToString(double Rate)
{
 wxString s;

    if(Rate<1000)
    {
        s.sprintf(wxT("%.1f B/s"),Rate);
    }
    else if(Rate<1000000)
    {
        s.sprintf(wxT("%.1f kB/s"),Rate/1000);
    }
    else
    {
        s.sprintf(wxT("%.1f MB/s"),Rate/1000000);
    }

    return s;
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 wxBusyCursor *BusyCursor;
 G69DDC_DEVICE_STATE DeviceState;
 wxLongLong Time;
 double Rate,Interval;
 wxString s;

    switch(Event.GetId())
    {
        case ID_STATE_TIMER:
            if(m_Receiver->GetDeviceState(&DeviceState))
            {
                Time=m_StopWatch.TimeInMicro();

                Interval=(double)(Time.GetValue()-m_Time.GetValue());
                Interval/=1000000; //convert us -> s

                if(Interval!=0)
                {
                    Rate=(double)(DeviceState.DataTransferred-m_BytesTransferred);
                    Rate/=Interval;
                    m_LDataRateValue->SetLabel(DataRateToString(Rate));

                    Rate=(double)(DeviceState.DataLost-m_BytesLost);
                    Rate/=Interval;
                    m_LLostDataRateValue->SetLabel(DataRateToString(Rate));

                    m_Time=Time;
                    m_BytesTransferred=DeviceState.DataTransferred;
                    m_BytesLost=DeviceState.DataLost;
                }

                if(m_Receiver->GetPower())
                {
                    if(DeviceState.Flags & G69DDC_DEVICE_STATE_HIGH_TEMPERATURE)
                    {
                        m_Receiver->SetPower(false);

                        wxMessageBox(wxT("WARNING: The receiver's temperature is too high.\n")
                                 wxT("The receiver has been turned off to prevent damage.\n\n")
                                 wxT("Please wait a moment and then turn the receiver on again.\n")
                                 wxT("If the problem persists, please ensure that the receiver's fan\n")
                                 wxT("and free air flow around the receiver are not obstructed."),wxT("Warning"),wxICON_WARNING|wxOK);
                    }
                }
            }

            if(!m_Receiver->IsConnected())
            {
                m_StateTimer.Stop();

                UpdateState();
                wxMessageBox(APP_TITLE wxT(" will be closed.\nReceiver is no longer connected."),wxT("Warning"),wxICON_WARNING|wxOK);
                Close();
            }
            break;

        case ID_INIT_POWER_ON_TIMER:
            BusyCursor=new wxBusyCursor;

            m_BPower->SetValue(true);
            m_BPower->Update();

            m_Receiver->SetPower(true);

            delete BusyCursor;
            break;
    }
}


void TMainWindow::UpdateDDC1RBWList(void)
{
 int i;
 int RBW;
 double RBWs[RBW_COUNT];

    m_CBDDC1RBW->Clear();

    m_Receiver->GetDDC1RBWs(RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=RBWs[i]+0.5;

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        m_CBDDC1RBW->Append(wxT(" ")+FormatFrequency(RBW));
    }

    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex());
}

void TMainWindow::UpdateDDC1RBW(void)
{
    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex());
}

void TMainWindow::UpdateDDC2(uint32_t ChannelIndex)
{
    if(m_Receiver->GetActiveChannel()==ChannelIndex)
    {
        m_DDC1Spectrum->SetBandWidth(m_Receiver->GetDDC2Info(ChannelIndex).Bandwidth);
    }
}

void TMainWindow::UpdateDDC2Frequency(uint32_t Channel)
{
    if(m_Receiver->GetActiveChannel()==Channel)
    {
        m_DDC1Spectrum->SetBandFrequency(m_Receiver->GetDDC2Frequency(Channel));
    }
}

void TMainWindow::UpdateFrontEndFrequency(void)
{
 uint64_t FrontEndFrequency;
 wxString s;

    FrontEndFrequency=m_Receiver->GetFrontEndFrequency();

    s.sprintf(wxT("%u MHz"),(uint32_t)(FrontEndFrequency/1000000));

    m_LFrontEndFrequencyValue->SetLabel(s);

    if(m_Receiver->GetRange()==Range_1)
    {
        m_IFSpectrum->SetCenterFrequency(m_DeviceInfo.Range1.Bandwidth/2);
    }
    else
    {
        m_IFSpectrum->SetCenterFrequency(FrontEndFrequency);
    }
}

void TMainWindow::UpdateRange(void)
{
 TRange Range;

    Range=m_Receiver->GetRange();

    if(Range==Range_1)
    {
        m_BPreamp->Hide();
        m_BMWFilter->Show();
        m_IFSpectrum->SetRange(m_DeviceInfo.Range1.Bandwidth);

        m_LRangeValue->SetLabel(wxT("1"));
    }
    else
    {
        m_BPreamp->Show();
        m_BMWFilter->Hide();
        m_IFSpectrum->SetRange(m_DeviceInfo.Range2.Bandwidth);

        m_LRangeValue->SetLabel(wxT("2"));
    }

    m_IFSizerBottom->Layout();
}

void TMainWindow::UpdateBottomPanel(void)
{
 bool IFSpectrum;
 bool Sweeper;

    IFSpectrum=m_BSpectrum->GetValue();
    Sweeper=!IFSpectrum;

    if(IFSpectrum)
    {
        m_IFSpectrum->Show(true);
        m_SwpSpectrum->Show(false);
    }
    else
    {
        m_SwpSpectrum->Show(true);
        m_IFSpectrum->Show(false);
    }

    m_CBIFRBW->Show(IFSpectrum);


    m_LDataRate->Show(IFSpectrum);
    m_LDataRateValue->Show(IFSpectrum);
    m_LLostDataRate->Show(IFSpectrum);
    m_LLostDataRateValue->Show(IFSpectrum);
    m_LFrontEndFrequency->Show(IFSpectrum);
    m_LFrontEndFrequencyValue->Show(IFSpectrum);
    m_LRange->Show(IFSpectrum);
    m_LRangeValue->Show(IFSpectrum);
    m_BIFUp->Show(IFSpectrum);
    m_BIFDown->Show(IFSpectrum);

    m_CBSwpRBW->Show(Sweeper);
    m_LSwpStart->Show(Sweeper);
    m_ESwpStart->Show(Sweeper);
    m_LSwpEnd->Show(Sweeper);
    m_ESwpEnd->Show(Sweeper);
    m_BSwpStart->Show(Sweeper);
    m_BSwpStop->Show(Sweeper);
    m_BSwpLoop->Show(Sweeper);
    m_BSwpClear->Show(Sweeper);

    Layout();
}

void TMainWindow::OnNumericEditChanged(wxNotifyEvent &Event)
{
 int64_t Freq;
 int64_t Max;

    switch(Event.GetId())
    {
        case ID_SWP_START_FREQ:
            Freq=((m_ESwpStart->Get()+1000000/2)/1000000)*1000000;

            Max=m_DeviceInfo.Range2.MaxFrequency+m_DeviceInfo.Range2.Bandwidth/2-1000000;

            if(Freq>Max)
            {
                Freq=Max;
            }

            if(Freq!=m_ESwpStart->Get())
            {
                m_ESwpStart->Set(Freq);
            }

            if(m_ESwpEnd->Get()<=Freq)
            {
                m_ESwpEnd->Set(Freq+1000000);
            }
            break;

        case ID_SWP_END_FREQ:
            Freq=((m_ESwpEnd->Get()+1000000/2)/1000000)*1000000;

            if(Freq<1000000)
            {
                Freq=1000000;
            }

            if(Freq!=m_ESwpEnd->Get())
            {
                m_ESwpEnd->Set(Freq);
            }

            if(m_ESwpStart->Get()>=Freq)
            {
                m_ESwpStart->Set(Freq-1000000);
            }
            break;
    }
}


void TMainWindow::UpdateSwpState(void)
{
 bool Enabled;
 bool Sweeper;

    Enabled=m_Receiver->GetPower() && m_Receiver->IsConnected();
    Sweeper=m_Receiver->GetSweeper();

    m_SwpSpectrum->Enable(Enabled);

    m_LRBW->Enable(Enabled && !Sweeper);
    m_CBSwpRBW->Enable(Enabled && !Sweeper);
    m_LSwpStart->Enable(Enabled && !Sweeper);
    m_ESwpStart->Enable(Enabled && !Sweeper);
    m_LSwpEnd->Enable(Enabled && !Sweeper);
    m_ESwpEnd->Enable(Enabled && !Sweeper);

    m_BSwpStart->Enable(Enabled);
    m_BSwpStart->SetValue(Sweeper);

    m_BSwpStop->Enable(Enabled && Sweeper);


    m_BSwpLoop->Enable(Enabled);
    m_BSwpClear->Enable(Enabled && !Sweeper);
}

void TMainWindow::OnStopSweeper(wxCommandEvent &Event)
{
    m_Receiver->StopSweeper();
}

bool TMainWindow::ReceiverCallback_SweeperQueryEnd(void)
{
    if(!m_SwpLoop)
    {
        wxCommandEvent Event(wxEVT_COMMAND_STOP_SWEEPER);

        //queue event which will stop sweeping in main thread
        AddPendingEvent(Event);
    }

    return !m_SwpLoop;
}

void TMainWindow::ReceiverCallbask_Sweeper(float *Data,uint32_t Count)
{
    m_SwpSpectrum->AddData(Data,Count);
}

