/* ***********************************************************************/
/*                                                                       */
/*                    WiNRADiO G69DDC EXCELSIOR ULTRA                    */
/*         Ioctl codes and definitions for G69DDC device driver          */
/*                     (c) Copyright 2022 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G69DDC_IOCTL_H__
#define __G69DDC_IOCTL_H__

#define INTERNAL_DRIVER_VERSION         1

#define BUS_TYPE_PCIE                   0
#define BUS_TYPE_USB2                   1
#define BUS_TYPE_USB3                   2

#define USB_VENDOR_ID                   0x14E0
#define USB_PRODUCT_ID                  0x1309

#define USB_LED_MODE_OFF                0
#define USB_LED_MODE_ON                 1
#define USB_LED_MODE_DIAG               2

#define USB_LED_PATTERN_OFF             0x0000
#define USB_LED_PATTERN_ON              0xFFFF
#define USB_LED_PATTERN_NO_DRIVER       0xFCCC
#define USB_LED_PATTERN_POWER_OFF       0x0033
#define USB_LED_PATTERN_POWER_ON        0xFFCC
#define USB_LED_PATTERN_APP_NOT_RUNNING 0x0333

#define USB_CRITICAL_TEMPERATURE_0      81
#define USB_CRITICAL_TEMPERATURE_1      75
#define USB_CRITICAL_TEMPERATURE_2      75

#pragma pack(push,1)

union g69ddc_state
{
    struct
    {
        uint32_t    connected:1;
        uint32_t    power:4;
        uint32_t    critical_temperature:1;
        uint32_t    suspended:1;
    } s;

    uint32_t    value;
};

struct g69ddc_ioctl_info
{
    uint16_t    driver_version;
    uint8_t     bus_type;
    uint16_t    vid_pid[4];
    uint16_t    fw[4];
};

struct g69ddc_ioctl_usb_ep
{
    uint8_t     address;
    uint16_t    max_packet_size;
    uint8_t     interval;
    uint8_t     type;
};

struct g69ddc_ioctl_usb_ctrl_vendor_msg
{
    uint8_t     request;
    uint16_t    value;
    uint16_t    index;
    void        *data;
    uint16_t    size;
};

struct g69ddc_ioctl_usb_bulk_msg
{
    uint32_t    ep_index;
    void        *data;
    uint32_t    size;
};

struct g69ddc_ioctl_usb_add_ep_buffer
{
    uint32_t    ep_index;
    void        *buffer;
    uint32_t    size;
};

struct g69ddc_ioctl_usb_ep_requeue_buffer
{
    uint32_t    ep_index;
    uint32_t    transfer_size;
};

union g69ddc_ioctl_usb_ep_complete_buffer
{
    uint32_t    ep_index;
    void        *buffer;
};

#pragma pack(pop)

#define G69DDC_IOCTL_GET_INFO                       _IOR(0x8C,0x00,struct g69ddc_ioctl_info)
#define G69DDC_IOCTL_GET_STATE                      _IOWR(0x8C,0x01,union g69ddc_state)
#define G69DDC_IOCTL_SET_POWER                      _IO(0x8C,0x02)

#define G69DDC_IOCTL_USB_GET_EP_COUNT               _IO(0x8C,0x10)
#define G69DDC_IOCTL_USB_GET_EP_INFO                _IOR(0x8C,0x11,struct g69ddc_ioctl_usb_ep)

#define G69DDC_IOCTL_USB_CTRL_VENDOR_MSG_RCV        _IOW(0x8C,0x17,struct g69ddc_ioctl_usb_ctrl_vendor_msg)
#define G69DDC_IOCTL_USB_CTRL_VENDOR_MSG_SND        _IOW(0x8C,0x18,struct g69ddc_ioctl_usb_ctrl_vendor_msg)
#define G69DDC_IOCTL_USB_BULK_MSG_RCV               _IOW(0x8C,0x19,struct g69ddc_ioctl_usb_bulk_msg)
#define G69DDC_IOCTL_USB_BULK_MSG_SND               _IOW(0x8C,0x1A,struct g69ddc_ioctl_usb_bulk_msg)
#define G69DDC_IOCTL_USB_SET_LED_MODE               _IO(0x8C,0x1B)

#define G69DDC_IOCTL_USB_EP_CLEAR_HALT              _IO(0x8C,0x30)
#define G69DDC_IOCTL_USB_EP_RESET                   _IO(0x8C,0x31)

#define G69DDC_IOCTL_USB_EP_ADD_BUFFER              _IOW(0x8C,0x33,struct g69ddc_ioctl_usb_add_ep_buffer)
#define G69DDC_IOCTL_USB_EP_CANCEL_BUFFERS          _IO(0x8C,0x34)
#define G69DDC_IOCTL_USB_EP_RELEASE_BUFFERS         _IO(0x8C,0x35)
#define G69DDC_IOCTL_USB_EP_REQUEUE_BUFFER          _IOW(0x8C,0x36,struct g69ddc_ioctl_usb_ep_requeue_buffer)
#define G69DDC_IOCTL_USB_EP_GET_COMPLETE_BUFFER     _IOWR(0x8C,0x37,union g69ddc_ioctl_usb_ep_complete_buffer)

#endif // __G69DDC_IOCTL_H__

